from abc import ABC, abstractmethod
from typing import TypeVar

from textual.app import ScreenStackError
from textual.reactive import var
from textual.screen import Screen

from systema.models.project import ProjectRead
from systema.proxies.base import Proxy


class BaseProjectScreen(ABC):
    @abstractmethod
    def get_proxy_type(self) -> type[Proxy[ProjectRead]]:
        pass


_Proxy = TypeVar("_Proxy", bound=Proxy)


class ProjectScreen(Screen[None]):
    project: var[ProjectRead | None] = var(None)

    def __init__(
        self,
        proxy_type: type[_Proxy],
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
    ) -> None:
        super().__init__(name, id, classes)
        self.proxy_type = proxy_type

    async def watch_project(self, project: ProjectRead | None):
        if project:
            self.sub_title = project.name
            self.proxy = self.proxy_type(project.id)
            await self.safe_refresh()

    @abstractmethod
    async def populate(self):
        raise NotImplementedError(
            f"Class {self.__class__.__name__} must implement method populate"
        )

    @abstractmethod
    async def clear(self):
        raise NotImplementedError(
            f"Class {self.__class__.__name__} must implement method clear"
        )

    async def on_mount(self):
        await self.safe_refresh()

    async def safe_refresh(self):
        if hasattr(self, "proxy"):
            await self.clear()
            await self.populate()

    def dismiss(self, result=None):
        try:
            return super().dismiss(result)
        except ScreenStackError:
            self.app.switch_mode("main")
